/*  TI-Terminal Driver for Linux                                     */
/*  Allows data to/from TI-8x to be redirected anywhere you want it  */
/*                                                                   */
/*  Michael Katz-Hyman (mkatshym@erols.com)                          */
/*  Parts taken from ti.c by David Ellsworth                         */
/*  Parts taken from Dos Term by Brian Rittmyer(sp?)                 */
 
#include <stdio.h>
#include <unistd.h>
#include <asm/io.h>
#include <termios.h>
#include <term.h>
#include <curses.h>

#define TIMAXTIME 10000
#define IO_DELAY 5 

int lpt_base=0x378;
int init_ti(int lpt_base);
int put_ti(int lpt_base, unsigned char data);
int probe(int lpt_base);
int get_ti(int lpt_base);
void init_keyboard();
void close_keyboard();
int kbdhit();
int readch();

/* Some stuff for the kbdhit() routine */

static struct termios initial_settings, new_settings;
static int peek_character = -1;

main() {
  int stop=0, key, ret;
  
  printf("Welcome TI-Term: Press ESC to exit\n");
  fflush(stdout);
  
  /* Ask permision to use the ports */
  
  if (ioperm(lpt_base,3,1)) {perror("ioperm");exit(1);}

  /* Initiate the terminal routines */

  init_keyboard();

  /* Initiate the link port */

  init_ti(lpt_base); 
  
  /* Let's get this party started */

  while (stop !=1) {
    if (kbdhit()) {
      key = readch();
      if (key == 27) {
	stop = 1;
	continue;
      }
      
      put_ti(lpt_base, key);
      printf("%c", key);
      if (key == 13) {
	printf("\n");
	fflush(stdout);
	continue;
      }
    
      fflush(stdout);
      continue;
    }  

    ret = get_ti(lpt_base);
    if (ret == -1) continue;
    if (ret == 13) {
      printf("\n");
      fflush(stdout);
      continue;
    }
    printf("%c",ret);
    fflush(stdout);
  
  }
  /* We don't need the ports anymore */

  if (ioperm(lpt_base,3,0)) {perror("ioperm");exit(1);}

  /* Or the keyboard */

  close_keyboard();

  fflush(stdin);
  return 0;
}

   
int init_ti(int lpt_base) {
  outb(3, lpt_base+0);
  return 0;
}

int put_ti(int lpt_base, unsigned char data) {
  int bit,i,titime=0;

  for (bit=0; bit<8; bit++) {
    for(i=0; i<IO_DELAY; i++) inb(lpt_base+1);
    
    if (data&1) {
      outb(2, lpt_base+0);
      while (inb(lpt_base+1)&0x10) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_base+0);
      while (!(inb(lpt_base+1)&0x10)) if(titime++>TIMAXTIME) return -1;
    } else {
      outb(1, lpt_base+0);
      while (inb(lpt_base+1)&0x20) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_base+0);
      while (!(inb(lpt_base+1)&0x20)) if(titime++>TIMAXTIME) return -1;
    }
    data>>=1;
  }
  return;
}

int get_ti(int lpt_base) {
  int bit,i,titime=0;
  unsigned char data=0, v;

  for (bit=0; bit<8; bit++) {
    while ((v=inb(lpt_base+1)&0x30)==0x30) if(titime++>TIMAXTIME) return -1;
    if (v==0x10) {
      data=(data>>1)|0x80;
      outb(1, lpt_base+0);
      while (!(inb(lpt_base+1)&0x20)) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_base+0);
    } else {
      data=data>>1;
      outb(2, lpt_base+0);
      while (!(inb(lpt_base+1)&0x10)) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_base+0);
    }
    for(i=0; i<IO_DELAY; i++) inb(lpt_base+1);
    
  }
  return (int)data;
}

void init_keyboard() {
  
  tcgetattr(0,&initial_settings);
  new_settings = initial_settings;
  new_settings.c_lflag &= ~ICANON;
  new_settings.c_lflag &= ~ECHO;
  new_settings.c_lflag &= ~ISIG;
  new_settings.c_cc[VMIN] = 1;
  new_settings.c_cc[VTIME] = 0;
  tcsetattr(0, TCSANOW, &new_settings);

}

void close_keyboard() {

  tcsetattr(0,TCSANOW, &initial_settings);
}

int kbdhit() {
  
  char ch;
  int nread;

  if(peek_character != -1)
    return 1;
  new_settings.c_cc[VMIN]=0;
  tcsetattr(0, TCSANOW, &new_settings);
  nread = read(0, &ch, 1);
  new_settings.c_cc[VMIN]=1;
  tcsetattr(0, TCSANOW, &new_settings);
  
  if(nread == 1) {
    peek_character = ch;
    return 1;
  }
  return 0;
}
  
int readch() {
  
  char ch;

  if(peek_character != -1) {
    ch = peek_character;
    peek_character = -1;
    return ch;
  }
  read(0,&ch,1);
  return ch;
}
    






















